--[[
AdiBags_TackleBox - Adds filters for the Fishing profession to your bags using the AdiBags inventory manager.
© 2017 Rigsby@Dalaran (upsidedownrabbit@gmail.com)
All rights reserved.
]]--

local addonName, addon = ...
local AdiBags = LibStub('AceAddon-3.0'):GetAddon('AdiBags')

local L = addon.L
local MatchIDs
local Tooltip

local function MatchIDs_Init(self)
	local Result = {}
	
	IDs = {
			60218,	-- Lure Master Tackle Box
			88535, 	-- Sharpened Tuskarr Spear
			117397,	-- Nat's Lucky Coin
	}
	AddToSet(Result, IDs)
	
	if self.db.profile.callPoles then
		IDs = {
			------- Fishing Poles
			6256, 	-- Fishing Pole
			6365, 	-- Strong Fishing Pole
			6366, 	-- Darkwood Fishing Pole
			6367,	-- Big Iron Fishing Pole
			6948,	-- Staats' Fishing Pole
			120163,	-- Thruk's Fishing Rod
			84660,	-- Pandaren Fishing Pole
			19022,	-- Nat Pagle's Extreme Angler FC-5000
			25978,	-- Seth's Graphite Fishing Pole
			45858,	-- Nat's Lucky Fishing Pole
			45991,	-- Bone Fishing Pole
			45992,	-- Jeweled Fishing Pole
			19970,	-- Arcanite Fishing Pole
			84661,	-- Dragon Fishing Pole
			118381,	-- Ephemeral Fishing Pole
			116825,	-- Savage Fishing Pole
			116826,	-- Draenic Fishing Pole
			44050,	-- Mastercraft Kalu'ak Fishing Pole
			133755,	-- Underlight Angler
		}
		AddToSet(Result, IDs)
	end
	
	if self.db.profile.callArmor then
		IDs = {
			------- Hats
			19972,	-- Lucky Fishing Hat
			33820,	-- Weather-Beaten Fishing Hat
			93732,	-- Darkmoon Fishing Cap
			118380,	-- Hightfish Cap
			118393,	-- Tentacled Hat
			88710,	-- Nat's Hat
			117405,	-- Nat's Drinking Hat
			------- Boots
			19969,	-- Nat Pagle's Extreme Anglin' Boots
			50287,	-- Boots of the Bay
		}
		AddToSet(Result, IDs)
	end
	
	if self.db.profile.callMiscGear then
		IDs = {
			------- Helmets
			10506,	-- Deepdive Helmet
			------- Gloves
			57481,	-- Fisherman's Gloves
			------- Belts
			57429,	-- Fisherman's Belt
			7052,	-- Azure Silk Belt
		}
		AddToSet(Result, IDs)
	end
	
	if self.db.profile.callTackle then
		IDs = {
			------- Lures
			67404,	-- Glass Fishing Bobber
			6529,	-- Shiny Bauble
			6530,	-- Nightcrawlers
			6811,	-- Aquadynamic Fish Lens
			6532,	-- Bright Baubles
			7307,	-- Flesh Eating Worm
			6533,	-- Aquadynamic Fish Attractor
			62673,	-- Feathered Lure
			34861,	-- Sharpened Fish Hook
			46006,	-- Glow Worm
			68049,	-- Heat-Treated Spinning Lure
			118391,	-- Arcane Lure
			------- Lines
			45194,	-- Tangled Fishing Line
			68796,	-- Reinforced Fishing Line
			34836,	-- Spun Truesilver Fishing Line
			19971,	-- High Test Eternium Fishing Line
			116117,	-- Rook's Lucky Fishin' Line
			------- Smears
			138961,	-- Alchemical Bonding Agent
		}
		AddToSet(Result, IDs)
	end
	
	if self.db.profile.callBait then
		IDs = {
			-------Draenor Bait
			110274,	--Jawless Skulker Bait
			110289,	--Fat Sleeper Bait
			110290,	--Blind Lake Sturgeon Bait
			110291,	--Fire Ammonite Bait
			110292,	--Sea Scorpion Bait
			110293,	--Abyssal Gulper Eel Bait
			110294,	--Blackwater Whiptail Bait
			-------Broken Isles Bait
			133701, --Skrog Toenail,
			133702, --Aromatic Murloc Slime,
			133703, --Pearlescent Conch,
			133704, --Rusty Queenfish Brooch,
			133705, --Rotten Fishbone,
			133706, --Mossgill Bait,
			133707, --Nightmare Nightcrawler,
			133708, --Drowned Thistleleaf,
			133709, --Funky Sea Snail,
			133710, --Salmon Lure,
			133711, --Swollen Murloc Egg,
			133712, --Frost Worm,
			133713, --Moosehorn Hook,
			133714, --Silverscale Minnow,
			133715, --Ancient Vrykul Ring,
			133716, --Soggy Drakescale,
			133717, --Enchanted Lure,
			133719, --Sleeping Murloc,
			133720, --Demonic Detritus,
			133721, --Message in a Beer Bottle,
			133722, --Axefish Lure,
			133723, --Stunned, Angry Shark,
			133724, --Decayed Whale Blubber,
		}
		AddToSet(Result, IDs)
	end
	
	if self.db.profile.callToys then
		IDs = {
			------- Bobbers
			136373,	-- Can O' Worms Bobber
			136374,	-- Toy Cat Head Bobber
			136375,	-- Squeaky Duck Bobber
			136376,	-- Murloc Bobber
			136377,	-- Oversized Bobber
			133688,	-- Tugboat Bobber
			142528,	-- Crate of Bobbers: Can of Worms
			142529,	-- Crate of Bobbers: Cat Head
			142530,	-- Crate of Bobbers: Tugboat
			142531,	-- Crate of Bobbers: Squeaky Duck
			142532,	-- Crate of Bobbers: Murloc Head
			143662,	-- Crate of Bobbers: Wooden Pepe
			------- Chairs, Rafts, & Other Toys
			85500,	-- Anglers Fishing Raft
			33223,	-- Fishing Chair
			86596,	-- Nat's Fishing Chair
			70161,	-- Mushroom Chair
			44430,	-- Titanium Seal of Dalaran
			86582,	-- Aqua Jewel
		}
		AddToSet(Result, IDs)
	end
	
	if self.db.profile.callJewelry then
		IDs = {
			------- Trinkets
			85973,	-- Ancient Pandaren Fishing Charm
			19979,	-- Hook of the Master Angler
		}
		AddToSet(Result, IDs)
	end
	
	if self.db.profile.callReadingMaterial then
		IDs = {
			------- Enchanting Formulae
			13620,	-- Formula: Enchant Gloves - Fishing
			71692,	-- Formula: Enchant Gloves - Angler
			------- Cooking Recipes
			34834,	-- Recipe: Captain Rumsey's Lager
			------- Books
			18229,  -- Nat Pagle's Guide to Extreme Anglin'
			18365,  -- A Thoroughly Read Copy of "Nat Pagle's Guide to Extreme Anglin'."
			139620, -- A Complete Copy of "Nat Pagle's Guide to Extreme Anglin'."
			34109,	-- Weather-Beaten Journal
			88563,	-- Nat's Fishing Journal
			117401,	-- Nat's Draenic Fishing Journal
			6307,	-- Message in a Bottle
			133721,	-- Message in a Beer Bottle
		}
		AddToSet(Result, IDs)
	end
	
	if self.db.profile.callProvisions then
		IDs = {
			------- Food
			110508,	-- "Fragrant" Pheromone Fish
			------- Drink
			34832,	-- Captain Rumsey's Lager
			------- Potions & Elixirs
			76096,	-- Darkwater Potion
			6372,	-- Swim Speed Potion
			8827,	-- Elixir of Water Walking
			5996,	-- Elixir of Water Breathing
			18294,	-- Elixir of Greater Water Breathing
			118711, -- Draenic Water Walking Elixir
			116271, -- Draenic Water Breathing Elixir
			44012,	-- Underbelly Elixir
		}
		AddToSet(Result, IDs)
	end
	
	return Result
end

local function Tooltip_Init()
	local Result, leftside = CreateFrame("GameTooltip"), {}
	for i = 1,6 do
		local L,R = Result:CreateFontString(), Result:CreateFontString()
		L:SetFontObject(GameFontNormal)
		R:SetFontObject(GameFontNormal)
		Result:AddFontStrings(L,R)
		leftside[i] = L
	end
	Result.leftside = leftside
	return Result
end

local function unescape(String)
	local Result = tostring(String)
	Result = gsub(Result, "|C........", "")		-- Remove color start.
	Result = gsub(Result, "|r","")				-- Remove color end.
	Result = gsub(Result, "|H.-|h(.-)|h", "%1")	-- Remove links.
	Result = gsub(Result, "|T.-|t", "")			-- Remove textures.
	Result = gsub(Result, "{.-}", "")			-- Remove raid icons.
	return Result
end

function AddToSet(Set, List)
	for _, v in ipairs(List) do
		Set[v] = true
	end
end

local setFilter = AdiBags:RegisterFilter("Tackle Box", 92, 'ABEvent-1.0')
setFilter.uiName = L["Tackle Box"]
setFilter.uiDesc = L["Fishing items grouped. This only applies to Fishing gear and equipment; Actual fish and other fished-up items will be extrapolated in another module."]

function setFilter:OnInitialize()
	self.db = AdiBags.db:RegisterNamespace("Tackle Box", {
		profile = {
			callPoles = true,
			callArmor = true,
			callMiscGear = false,
			callTackle = true,
			callBait = true,
			callToys = true,
			callJewelry = true,
			callReadingMaterial = false,
			callProvisions = false
		},
		char = {  },
	})
end

function setFilter:Update()
	MatchIDs = nil
	self:SendMessage('AdiBags_FiltersChanged')
end

function setFilter:OnEnable()
	AdiBags:UpdateFilters()
end

function setFilter:OnDisable()
	AdiBags:UpdateFilters()
end

function setFilter:Filter(slotData)
	MatchIDs = MatchIDs or MatchIDs_Init(self)
	if MatchIDs[slotData.itemId] then
		return L["Tackle Box"]
	end
		Tooltip = Tooltip or Tooltip_Init()
		Tooltip:SetOwner(UIParent, "ANCHOR_NONE")
		Tooltip:ClearLines()
		if slotData.bag == BANK_CONTAINER then
			Tooltip:SetInventoryItem("player", BankButtonIDToInvSlotID(slotData.slot, nil))
		else
			Tooltip:SetBagItem(slotData.bag, slotData.slot)
		end
		
		Tooltip:Hide()
	end

function setFilter:GetOptions()
	return {
		callPoles = {
			name	= L["Fishing Poles"],
			desc	= L["Add fishing poles to this group."],
			type	= 'toggle',
			width	= 'double',
			order	= 10,
		},
		callArmor = {
			name	= L["Hats & Boots"],
			desc	= L["Add gear such as Lucky Fishing Hat or Boots of the Bay to this group."],
			type	= 'toggle',
			width	= 'double',
			order	= 20,
		},
		callMiscGear = {
			name	= L["Other Fishing Gear"],
			desc	= L["Show other apparel that may be useful for Fishing in this group, such as Deepdive Helmet or Azure Silk Belt."],
			type	= 'toggle',
			width	= 'double',
			order	= 30,
		},
		callTackle = {
			name	= L["Tackle"],
			desc	= L["Add items such as lures and fishing lines to this group."],
			type	= 'toggle',
			width	= 'double',
			order	= 40,
		},
		callBait = {
			name	= L["Bait"],
			desc	= L["Add bait for fishing in Draenor and the Broken Isles to this group."],
			type	= 'toggle',
			width	= 'double',
			order	= 50,
		},
		callToys = {
			name	= L["Toys"],
			desc	= L["Add specialty toys such as bobbers and fishing chairs to this group."],
			type	= 'toggle',
			width	= 'double',
			order	= 60,
		},
		callJewelry = {
			name	= L["Jewelry"],
			desc	= L["Show items such as Pandaren Fishing Charm or Hook of the Master Angler in this group."],
			type	= 'toggle',
			width	= 'double',
			order	= 70,
		},
		callReadingMaterial = {
			name	= L["Books, Scrolls, & Recipes"],
			desc	= L["Add fishing journals and related cooking recipes or enchanting formulae to this group."],
			type	= 'toggle',
			width	= 'double',
			order	= 80,
		},
		callProvisions = {
			name	= L["Provisions"],
			desc	= L["Add consumables such as Captain Rumsey's Lager or Elixir of Water Walking to this group."],
			type	= 'toggle',
			width	= 'double',
			order	= 90,
		},
	},	AdiBags:GetOptionHandler(self, false, function() return self:Update() end)
end